#include <stdio.h>		/* for NULL */
#include <stdlib.h>
#include "ll.h"


	/*
	 * singly linked list 
	 */
typedef struct anode {
    char *line;
    struct anode *prev;
    struct anode *next;
} node;

typedef struct alist {
    node *head;
    node *current;
    node *last;
} list;

/*
 * @null@
 */
void *
ll_newlist ()
{
    list *x;

    x = (list *) malloc (sizeof (list));
    if (x != NULL)
    {
	x->head = NULL;
	x->current = NULL;
	x->last = NULL;
	return x;
    }
    printf ("Failed to gnerate new list");
    exit (-1);
    return NULL;
}

int
ll_next (void *this)
{
    list *x;

    x = (list *) this;
    if (x == NULL || x->current == NULL || x->current->next == NULL)
	return 0;
    x->current = x->current->next;
    return 1;
}

int
ll_prev (void *this)
{
    list *x;

    x = (list *) this;
    if (x == NULL || x->current == NULL || x->current->prev == NULL)
	return 0;
    x->current = x->current->prev;
    return 1;
}

int
ll_rewind (void *this)
{
    list *x;

    if (this == NULL)
	return 0;
    x = (list *) this;
    x->current = x->head;
    return 1;
}

int
ll_addatend (void *this, char *line)
{
    node *x;
    list *l;

    if (this == NULL || line == NULL)	/* bad list */
	return 0;

    /*
     * generate node 
     */
    x = (node *) malloc (sizeof (node));
    if (x == NULL)
	return 0;

    x->line = (char *) malloc (sizeof (char) * (strlen (line) + 1));
    if (x->line == NULL)
	return 0;		/* no malloc */

    strcpy (x->line, line);
    x->next = NULL;

    l = (list *) this;

    x->prev = l->last;

    /*
     * attach node to list 
     */
    if (l->head == NULL)
	l->head = l->current = l->last = x;
    else
    {
	l->last->next = x;
	l->last = x;
    }

    return 1;
}

char *
ll_showline (void *this)
{
    list *l;

    l = (list *) this;
    if (l == NULL || l->current == NULL || l->current->line == NULL)
	return NULL;
    return l->current->line;
}

extern char *
ll_shownext (void *this)
{
    list *l;

    l = (list *) this;
    if (l == NULL || l->current == NULL || l->current->next == NULL || l->current->next->line == NULL)
	return NULL;
    return l->current->next->line;
}

extern char *
ll_showprev (void *this)
{
    list *l;

    l = (list *) this;
    if (l == NULL || l->current == NULL || l->current->prev == NULL || l->current->prev->line == NULL)
	return NULL;
    return l->current->prev->line;
}

static void
ll_rec_clearlist (node * this)
{
    if (this == NULL)
	return;
    if (this->next != NULL)
	ll_rec_clearlist (this->next);
    free (this->line);
    free (this);
    return;
}


int
ll_clearlist (void *this)
{

    /*
     * node *x, *y; 
     */
    if (this == NULL || ((list *) this)->head == NULL)
	return 0;

    /*
     * x = ((list *) this)->head; while (x->next != NULL) { y = x->next; if
     * (x->line != NULL) free (x->line); free (x); x = y; } 
     */
    ll_rec_clearlist (((list *) this)->head);
    free (this);
    return 1;
}

int
ll_empty (void *v)
{
    list *l;

    l = (list *) v;
    if (l == NULL || l->head == NULL)
	return 1;
    return 0;
}

int
ll_deletenode (void *v)
{
    list *l;
    node *n, *t;

    l = (list *) v;
    if (l == NULL || l->head == NULL)
	return 0;
    n = l->head;

    while (n->next != NULL && n->next != l->current)
	n = n->next;

    if (n->next == NULL)
	return 0;

    t = n->next;
    n->next = t->next;
    free (t);
    l->current = n->next;

    return 1;
}

int
ll_beginning (void *n)
{
    list *l;

    l = (list *) n;
    if (l->current == l->head)
	return 1;
    return 0;
}

int
ll_end (void *n)
{
    list *l;

    l = (list *) n;
    if (l->current == NULL || l->current->next == NULL)
	return 1;
    return 0;
}

int
ll_addatbeginning (void *this, char *line)
{
    node *x;
    list *l;

    if (this == NULL || line == NULL)	/* bad list */
	return 0;

    /*
     * generate node 
     */
    x = (node *) malloc (sizeof (node));
    if (x == NULL)
	return 0;

    x->line = (char *) malloc (sizeof (char) * (strlen (line) + 1));
    if (x->line == NULL)
	return 0;		/* no malloc */

    strcpy (x->line, line);
    x->next = NULL;

    l = (list *) this;

    x->next = l->head;

    /*
     * attach node to list 
     */
    if (l->head == NULL)
	l->head = l->current = l->last = x;
    else
    {
	l->head->prev = x;
	l->head = x;
    }

    return 1;
}

	/*** TODO ***/
int
ll_addbeforecurrent (void *this, char *line)
{
    node *x;
    list *l;

    if (this == NULL || line == NULL)	/* bad list */
	return 0;

    /*
     * generate node 
     */
    x = (node *) malloc (sizeof (node));
    if (x == NULL)
	return 0;

    x->line = (char *) malloc (sizeof (char) * (strlen (line) + 1));
    if (x->line == NULL)
	return 0;		/* no malloc */

    strcpy (x->line, line);
    x->next = NULL;

    l = (list *) this;

    x->prev = l->last;

    /*
     * attach node to list 
     */
    if (l->head == NULL)
	l->head = l->current = l->last = x;
    else
    {
	l->last->next = x;
	l->last = x;
    }

    return 1;
}

	/*** TODO ***/
int
ll_addaftercurrent (void *this, char *line)
{
    node *x;
    list *l;

    if (this == NULL || line == NULL)	/* bad list */
	return 0;

    /*
     * generate node 
     */
    x = (node *) malloc (sizeof (node));
    if (x == NULL)
	return 0;

    x->line = (char *) malloc (sizeof (char) * (strlen (line) + 1));
    if (x->line == NULL)
	return 0;		/* no malloc */

    strcpy (x->line, line);
    x->next = NULL;

    l = (list *) this;

    x->prev = l->last;

    /*
     * attach node to list 
     */
    if (l->head == NULL)
	l->head = l->current = l->last = x;
    else
    {
	l->last->next = x;
	l->last = x;
    }

    return 1;
}
